@@ -928,7 +928,7 @@ find_toplevel_window (int depth, Window xid, gboolean *keep_going)
 	return 0;
 }
 
-static void
+static gboolean
 take_window_shot (void)
 {
 	GdkWindow *window;
@@ -958,6 +958,8 @@ take_window_shot (void)
 	} else {
 
                 window = gdk_window_foreign_new (child);
+		if (window == NULL)
+			return FALSE;
 
 		keep_going = TRUE;
 
@@ -998,6 +1000,8 @@ take_window_shot (void)
 						   width, height);
 
 	class_name = result;
+
+	return TRUE;
 }
 
 static void
@@ -1061,6 +1065,61 @@ drag_begin (GtkWidget *widget, GdkDragContext *context)
 	start_temporary ();
 }
 
+/* To make sure there is only one screenshot taken at a time,
+ * (Imagine key repeat for the print screen key) we hold a selection
+ * until we are done taking the screenshot
+ */
+static GtkWidget *selection_window;
+
+#define SELECTION_NAME "_GNOME_PANEL_SCREENSHOT"
+
+static gboolean
+get_lock (void)
+{
+        Atom selection_atom = gdk_x11_get_xatom_by_name (SELECTION_NAME);
+	GdkCursor *cursor;
+	gboolean result = FALSE;
+
+	XGrabServer (GDK_DISPLAY ());
+        if (XGetSelectionOwner (GDK_DISPLAY(), selection_atom) != None)
+                goto out;
+
+	selection_window = gtk_invisible_new ();
+	gtk_widget_show (selection_window);
+
+	if (!gtk_selection_owner_set (selection_window,
+				      gdk_atom_intern (SELECTION_NAME, FALSE),
+				      GDK_CURRENT_TIME)) {
+		gtk_widget_destroy (selection_window);
+		selection_window = NULL;
+		goto out;
+	}
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_pointer_grab (selection_window->window, FALSE, 0, NULL,
+			  cursor, GDK_CURRENT_TIME);
+	gdk_cursor_unref (cursor);
+
+	result = TRUE;
+
+ out:
+	XUngrabServer (GDK_DISPLAY ());
+	gdk_flush ();
+
+        return result;
+}
+
+static void
+release_lock (void)
+{
+	if (selection_window) {
+		gtk_widget_destroy (selection_window);
+		selection_window = NULL;
+	}
+
+	gdk_flush ();
+}
+
 /* main */
 int
 main (int argc, char *argv[])
@@ -1099,11 +1158,20 @@ main (int argc, char *argv[])
 	if (delay > 0) {
 		sleep (delay);	
 	}
+
+	if (!get_lock ()) {
+		g_printerr ("gnome-panel-screenshot is already running\n");
+		exit (1);
+	}
 	
-	if (window)
-		take_window_shot ();
-	else
+	if (window) {
+		if ( ! take_window_shot ()) {
+			release_lock ();
+			exit (1);
+		}
+	} else {
 		take_screen_shot ();
+	}
 
 	if (g_file_test ("gnome-panel-screenshot.glade", G_FILE_TEST_EXISTS)) {
 		xml = glade_xml_new ("gnome-panel-screenshot.glade", NULL, NULL);
@@ -1114,6 +1182,7 @@ main (int argc, char *argv[])
 	}
 	if (xml == NULL) {
 		GtkWidget *dialog;
+		release_lock ();
 		dialog = gtk_message_dialog_new
 			(NULL /* parent */,
 			 0 /* flags */,
@@ -1137,6 +1206,7 @@ main (int argc, char *argv[])
 
 	if (screenshot == NULL) {
 		GtkWidget *dialog;
+		release_lock ();
 		dialog = gtk_message_dialog_new
 			(NULL /* parent */,
 			 0 /* flags */,
@@ -1218,6 +1288,7 @@ main (int argc, char *argv[])
 			  NULL);
 
 	gtk_widget_show_now (toplevel);
+	release_lock ();
 
 	/*
 	 * Start working on the temporary file in a fork, now this is
