@@ -46,8 +46,7 @@
 #include "panel-gconf.h"
 #include "quick-desktop-reader.h"
 #include "nothing.h"
-
-#define ICON_SIZE 20
+#include "panel-stock-icons.h"
 
 enum {
 	COLUMN_ICON,
@@ -63,8 +62,6 @@ typedef enum {
 	PANEL_RESPONSE_RUN
 } PanelResponseType;
 
-#define PANEL_STOCK_RUN "panel-run"
-
 #define ENABLE_LIST_DEFAULT TRUE
 #define SHOW_LIST_DEFAULT FALSE
 
@@ -272,6 +269,7 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 	int argc, temp_argc;
 	char *s = NULL;
 	char *escaped = NULL;
+	char *disk = NULL;
 	char **envv = NULL;
 	int envc;
 	GError *error = NULL;
@@ -305,11 +303,13 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
                 entry = g_object_get_data (G_OBJECT (w), "entry");
 
                 s = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-		escaped = g_markup_escape_text (s, -1);
 
                 if (string_empty (s))
                         goto return_and_close;
 
+		escaped = g_markup_escape_text (s, -1);
+		disk = g_filename_from_utf8 (s, -1, NULL, NULL, NULL);
+
 		/* save command in history */
 		gnome_entry = g_object_get_data (G_OBJECT (w), "gnome_entry");
 		gnome_entry_prepend_history (gnome_entry, TRUE, s);
@@ -341,11 +341,15 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 
                 /* Somewhat of a hack I suppose */
                 if (panel_is_url (s)) {
+			/* FIXME: URLs are in UTF8 ... right? */
                         gnome_url_show (s, NULL);
                         goto return_and_close;
                 }
 
-                if ( ! g_shell_parse_argv (s, &temp_argc, &temp_argv, &error)) {
+		/* Note, the command is taken to have to be in disk encoding
+		 * even though it could contain strings, but more likely
+		 * it is all filenames and thus should be in disk encoding */
+                if ( ! g_shell_parse_argv (disk, &temp_argc, &temp_argv, &error)) {
 			panel_error_dialog ("run_error",
 					    _("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
 					    escaped, error->message);
@@ -475,6 +479,7 @@ return_and_close:
 	g_strfreev (envv);
 	g_free (s);
 	g_free (escaped);
+	g_free (disk);
 
 	gtk_widget_destroy (w);
         
@@ -497,7 +502,7 @@ quote_string (const char *s)
 }
 
 static void
-append_file (GtkWidget *entry, const char *file)
+append_file_utf8 (GtkWidget *entry, const char *file)
 {
 	const char *text;
 	char *quoted = quote_string (file);
@@ -512,6 +517,16 @@ append_file (GtkWidget *entry, const char *file)
 	g_free (quoted);
 }
 
+static void
+append_file (GtkWidget *entry, const char *file)
+{
+	char *utf8_file = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+	if (utf8_file != NULL) {
+		append_file_utf8 (entry, utf8_file);
+		g_free (utf8_file);
+	}
+}
+
 static void
 browse_ok (GtkWidget *widget, GtkFileSelection *fsel)
 {
@@ -849,11 +864,13 @@ drag_data_received (GtkWidget        *widget,
 	for (i = 0; uris[i] != NULL; i++) {
 		char *file = gnome_vfs_get_local_path_from_uri (uris[i]);
 
+		/* FIXME: I assume the file is in utf8 encoding if coming
+		 * from a URI? */
 		if (file != NULL) {
-			append_file (entry, file);
+			append_file_utf8 (entry, file);
 			g_free (file);
 		} else {
-			append_file (entry, uris[i]);
+			append_file_utf8 (entry, uris[i]);
 		}
 	}
 
@@ -1145,6 +1162,7 @@ add_icon_idle (GtkListStore *list)
 	gboolean     long_operation = FALSE;
 	GdkPixbuf   *pixbuf;
 	char        *file;
+	int          icon_height;
 
 	do {
 		if (add_icon_paths == NULL) {
@@ -1168,8 +1186,10 @@ add_icon_idle (GtkListStore *list)
 		gtk_tree_model_get (GTK_TREE_MODEL (list), &iter,
 				    COLUMN_ICON_FILE, &file, -1);
 
-		pixbuf = panel_make_menu_icon (file, NULL /* fallback */,
-					       ICON_SIZE, &long_operation);
+		if (!gtk_icon_size_lookup (panel_menu_icon_get_size (), NULL, &icon_height))
+			icon_height = PANEL_DEFAULT_MENU_ICON_SIZE;
+
+		pixbuf = panel_make_menu_icon (file, NULL, icon_height, &long_operation);
 		if (pixbuf) {
 			gtk_list_store_set (list, &iter, COLUMN_ICON, pixbuf, -1);
 			g_object_unref (pixbuf);
@@ -1454,8 +1474,9 @@ create_program_list_contents (void)
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list), FALSE);
 
-	add_atk_name_desc (list, _("List of known applications"),
-			   _("Choose an application to run from the list"));
+	panel_set_atk_name_desc (list,
+				 _("List of known applications"),
+				 _("Choose an application to run from the list"));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -1539,33 +1560,6 @@ update_contents (GtkWidget *dialog)
         }
 }
 
-static inline void
-register_run_stock_item (void)
-{
-	static gboolean registered = FALSE;
-
-	if (!registered) {
-		GtkIconFactory      *factory;
-		GtkIconSet          *execute_icons;
-
-		static GtkStockItem  run_item [] = {
-			{ PANEL_STOCK_RUN, N_("_Run"), 0, 0, GETTEXT_PACKAGE },
-		};
-
-		execute_icons = gtk_icon_factory_lookup_default (GTK_STOCK_EXECUTE);
-
-		factory = gtk_icon_factory_new ();
-
-		gtk_icon_factory_add (factory, PANEL_STOCK_RUN, execute_icons);
-
-		gtk_icon_factory_add_default (factory);
-
-		gtk_stock_add_static (run_item, 1);
-
-		registered = TRUE;
-	}
-}
-
 static void
 run_dialog_destroyed (GtkWidget *widget)
 {
@@ -1606,8 +1600,6 @@ show_run_dialog (void)
 		return;
 	}
 
-	register_run_stock_item ();
-
 	run_dialog = gtk_dialog_new_with_buttons (_("Run Program"),
 						  NULL /* parent */,
 						  0 /* flags */,
@@ -1620,7 +1612,7 @@ show_run_dialog (void)
 	gtk_window_set_resizable (GTK_WINDOW (run_dialog), FALSE);
 
 	w = gtk_dialog_add_button (GTK_DIALOG (run_dialog),
-				   PANEL_STOCK_RUN, PANEL_RESPONSE_RUN);
+				   PANEL_STOCK_EXECUTE, PANEL_RESPONSE_RUN);
 	gtk_widget_set_sensitive (w, FALSE);
 	g_object_set_data (G_OBJECT (run_dialog), "run_button", w);
 
