@@ -107,29 +107,21 @@ struct _PanelWidget
 						   was set, this is used
 						   for tiling onto the
 						   background */
-	GdkEventKey		*key_event;
 };
 
 struct _PanelWidgetClass
 {
 	GtkFixedClass parent_class;
 
-	void (* orient_change) (PanelWidget *panel,
-				GtkOrientation orient);
-	void (* size_change) (PanelWidget *panel,
-			      int sz);
+	void (* orient_change) (PanelWidget *panel);
+	void (* size_change) (PanelWidget *panel);
 	void (* applet_move) (PanelWidget *panel,
 			      GtkWidget *applet);
 	void (* applet_added) (PanelWidget *panel,
 			       GtkWidget *applet);
 	void (* applet_removed) (PanelWidget *panel,
 				 GtkWidget *applet);
-	void (* back_change) (PanelWidget *panel,
-			      PanelBackType type,
-			      char *pixmap,
-			      GdkColor *color);
-	void (* applet_about_to_die) (PanelWidget *panel,
-				      GtkWidget *applet);
+	void (* back_change) (PanelWidget *panel);
 	void (* push_move) (PanelWidget		*panel,
                             GtkDirectionType	 dir);
 	void (* switch_move) (PanelWidget	*panel,
@@ -139,6 +131,7 @@ struct _PanelWidgetClass
 	void (* tab_move) (PanelWidget	*panel,
                            gboolean	 next);
 	void (* end_move) (PanelWidget	*panel);
+	gboolean (* popup_panel_menu) (PanelWidget	*panel);
 
 };
 
@@ -231,10 +224,6 @@ void            panel_widget_focus              (PanelWidget *panel);
 
 PanelOrient     panel_widget_get_applet_orient  (PanelWidget *panel);
 
-void            panel_widget_save_key_event	(PanelWidget	*panel,
-                                                 GdkEventKey	*key_event);
-
-
 extern gboolean panel_applet_in_drag;
 
 G_END_DECLS
